%% Fast Radial Symetry Transform
% usage: call the function "frst1"
% S = frst1 (im, radii, alpha)

% input arguments


%       im    ... input image
%       radii ... vector of radius or radii
%       alpha ... parameter alpha according to the article is between 1-3
%                 and represents the strictness of the radial objects
%                 1 - least strict; 3 - most strict

%% Script

function [S,centers,bestIdx] = frst1 (im, radii, alpha)

[rows, cols] = size(im);


% image gradient using the gradient function
[imgx,imgy] = gradient (double(im),2);

norm = sqrt(imgx.^2 + imgy.^2)+eps;
imgx = imgx./norm;
imgy = imgy./norm;

S = zeros(rows,cols);  % Symmetry matrix

[x,y] = meshgrid(1:cols, 1:rows);
MAX = zeros (1,length(radii));
centers = zeros(length(radii),2);
k = 1;

for n = radii
    M = zeros(rows,cols);  % Magnitude projection image
    O = zeros(rows,cols);  % Orientation projection image
    
    % Coordinates of positively and negatively affected pixels
    posx = x + round(n*imgx);
    posy = y + round(n*imgy);
    
    negx = x - round(n*imgx);
    negy = y - round(n*imgy);
    
    % Positioning the all the affected pixels into the matrix
    posx( posx<1 )    = 1;
    posx( posx>cols ) = cols;
    posy( posy<1 )    = 1;
    posy( posy>rows ) = rows;
    
    negx( negx<1 )    = 1;
    negx( negx>cols ) = cols;
    negy( negy<1 )    = 1;
    negy( negy>rows ) = rows;
    
    % subscripts to logical indexing
    pos = sub2ind([rows cols], posy, posx);
    neg = sub2ind([rows cols], negy, negx);
    
    % matrix to vector
    pos = pos (:);
    neg = neg (:);
    
    % orientation & magnitute projections
    % incrementing the matrices for in all positive a negative pixels
    O(:) = full(sparse([pos,neg], 1, [ones(rows,cols),-ones(rows,cols)]));
    M(:) = full(sparse([pos,neg], 1, [norm,-norm]));
    
    % deleting the all the pixels pointing out of the input image
    O(1,:) = 0;
    O(rows,:) = 0;
    O(:,1) = 0;
    O(:,cols) = 0;
    
    M(1,:) = 0;
    M(rows,:) = 0;
    M(:,1) = 0;
    M(:,cols) = 0;
    
    if max([pos,neg]) ~= length(pos)       
        O((max([pos,neg])+1):length(pos)) = 0;
        M((max([pos,neg])+1):length(pos)) = 0; 
    end
    
    % Set the orientation matrix into the of -kappa +kappa according to
    % the article suggestion
    
    if n == 1  
        kappa = 8;  
    else  
        kappa = 9.9;  
    end
    
    O(O >  kappa) =  kappa;
    O(O < -kappa) = -kappa;
    
    % calculation of F
    F = M./kappa .* (abs(O)/kappa).^alpha;
    
    % creation of weighted gaussian kernel A
    %         A = fspecial('gaussian',[rows cols], 0.25*n) * n;
    A = fspecial('gaussian',[n n], 0.25*n) * n;
    
    % convolution which calculates the symmetry map
    %         S1 = ifft2(fft2(F).*fft2(A));
    %         S1 = fftshift(S1);
    S1 = imfilter(F,A);
    S(:,:,k) = S1;
    [C,I] = max(S1(:));
    MAX(k) = C;
    [centers(k,1) centers(k,2)] = ind2sub(size(S1),I);
    k = k + 1;
end

% Normalizing the symmetry map according to the number of different
% radii

% finding the maximum of the symmetry map and localizing the centers
[~,bestIdx] = max (MAX);








