%% Parabola Function
function [X,Y] = doubleParabolaPoints( a , b , xBounds , yBounds, step)

% Check for correct a
if a<0
   disp('Error: a has to be positive'); 
end

% Bring beta in range [-pi,pi]
while (b>pi/2)
    b=b-pi;
end
while (b<=-pi/2)
    b=b+pi;
end

% Precompute trigonometric functions
si = sin(b);
co = cos(b);
ta = tan(b);

xmin = xBounds(1);
xmax = xBounds(2);
ymin = yBounds(1);
ymax = yBounds(2);

xi=xmin:step:xmax;
yi=ymin:step:ymax;

X=[];
Y=[];

% Case-by-case analysis for absolute value function:

if (abs(b)==pi/2) % case 1: cos(b)==0
    
    yInd = find(yi>=0);
    X = [X, sqrt(yi(yInd)/a), -sqrt(yi(yInd)/a)];
    Y = [Y, yi(yInd), yi(yInd)];
    
    yInd = find(yi<0);
    X = [X, sqrt(-yi(yInd)/a), -sqrt(-yi(yInd)/a)];
    Y = [Y, yi(yInd), yi(yInd)];
    
    X = [X, xi, xi];
    Y = [Y, a*xi.^2, a*xi.^2];

elseif (abs(b)<pi/2) % CASE 2: cos(b)>0

    % case 2.1.x: determine x from given y with x*cos(b)-y*sin(b) >= 0
    [Y1, X1] = solveQuadraticFunc(yi, a*si^2, 2*a*yi*si*co - co, a*yi.^2*co^2 + yi*si);
    xInd = find((X1-Y1*ta)>=0);
    X=[X,X1(xInd)];
    Y=[Y,Y1(xInd)];
    
    % case 2.1.y: determine y from given x with x*cos(b)-y*sin(b) >= 0
    [X1, Y1] = solveQuadraticFunc(xi, a*co^2, 2*a*xi*si*co + si, a*xi.^2*si^2 - xi*co);
    yInd = find((X1-Y1*ta)>=0);
    X=[X,X1(yInd)];
    Y=[Y,Y1(yInd)];
  
    % case 2.2.x: determine x from given y with x*cos(b)-y*sin(b) < 0
    [Y1, X1] = solveQuadraticFunc(yi, a*si^2, 2*a*yi*si*co + co, a*yi.^2*co^2 - yi*si);
    xInd = find((X1-Y1*ta)<0);
    X=[X,X1(xInd)];
    Y=[Y,Y1(xInd)];
    
    % case 2.2.y: determine y from given x with x*cos(b)-y*sin(b) < 0
    [X1, Y1] = solveQuadraticFunc(xi, a*co^2, 2*a*xi*si*co - si, a*xi.^2*si^2 + xi*co);
    yInd = find((X1-Y1*ta)<0);
    X=[X,X1(yInd)];
    Y=[Y,Y1(yInd)];
    
elseif (abs(b)>pi/2) % case 3: cos(b)<0
    
    % case 3.1.x: determine x from given y with x*cos(b)-y*sin(b) >= 0
    [Y1, X1] = solveQuadraticFunc(yi, a*si^2, 2*a*yi*si*co - co, a*yi.^2*co^2 + yi*si);
    xInd = find((X1-Y1*ta)<=0);
    X=[X,X1(xInd)];
    Y=[Y,Y1(xInd)];
    
    % case 3.1.y: determine y from given x with x*cos(b)-y*sin(b) >= 0
    [X1, Y1] = solveQuadraticFunc(xi, a*co^2, 2*a*xi*si*co + si, a*xi.^2*si^2 - xi*co);
    yInd = find((X1-Y1*ta)<=0);
    X=[X,X1(yInd)];
    Y=[Y,Y1(yInd)];
  
    % case 3.2.x: determine x from given y with x*cos(b)-y*sin(b) < 0
    [Y1, X1] = solveQuadraticFunc(yi, a*si^2, 2*a*yi*si*co + co, a*yi.^2*co^2 - yi*si);
    xInd = find((X1-Y1*ta)>0);
    X=[X,X1(xInd)];
    Y=[Y,Y1(xInd)];
    
    % case 3.2.y: determine y from given x with x*cos(b)-y*sin(b) < 0
    [X1, Y1] = solveQuadraticFunc(xi, a*co^2, 2*a*xi*si*co - si, a*xi.^2*si^2 + xi*co);
    yInd = find((X1-Y1*ta)>0);
    X=[X,X1(yInd)];
    Y=[Y,Y1(yInd)];
  
end

i = find((X<=xmax) .* (X>=xmin) .* (Y<=ymax) .* (Y>=ymin));
X = X(i);
Y = Y(i);

end

%% Helper Function
function [X,Y] = solveQuadraticFunc (x, c1,c2,c3)

    radicand = c2.^2 - 4*c1*c3;
    xInd = find(radicand>=0);
    
    y1 = (-c2(xInd)+sqrt(radicand(xInd)))/(2*c1);
    y2 = (-c2(xInd)-sqrt(radicand(xInd)))/(2*c1);
    
    X = [x(xInd),x(xInd)];    
    Y = [y1,y2]; 
end
