close all;
clear all;

%% Read Data
scale = 0.2;
vesselImage = imread( 'Data\vessels\01_g.tif');
vesselImage = imresize(vesselImage, scale);
vesselImage(vesselImage > 0) = 1;
vesselImage = uint8(vesselImage);
[m,n] = size(vesselImage);

vCenter = [2773; 1096] * scale

%% Preprocessing: 
%Fill holes
CC = bwconncomp(1-vesselImage,4);
numPixels = cellfun(@numel,CC.PixelIdxList);
ind = find(numPixels<10);
pixels = CC.PixelIdxList(ind);
vesselImage(cell2mat(pixels(:))) = 1;

distTrafo = bwdist(1-vesselImage,'cityblock');

% Preprocessing: Determine centerline 
% skeletonize -> enhance skeleton (remove border, fill holes)
skeleton = bwmorph(vesselImage,'skel',Inf);
skeleton([1,end],:) = 0;
skeleton(:,[1,end]) = 0;
skeletonFilled = imerode(skeleton,[0,1,0;1,0,1;0,1,0]) + skeleton;
skeletonFilled = bwmorph(skeletonFilled,'majority') + skeletonFilled;
skeleton = bwmorph(skeletonFilled,'skel',Inf);
    
branchpoints = bwmorph(skeleton,'branchpoints');
endpoints = bwmorph(skeleton,'endpoints');
poi = (branchpoints+endpoints)>0;
%poi = bwmorph(poi,'shrink',Inf);

result(:,:,1) = (skeleton+poi)>0;
result(:,:,2) = (poi)>0;
result(:,:,3) = (poi)>0;
result = result*255;  

%% Compute graph

poiCross = imdilate(poi,[0,1,0;1,1,1;0,1,0]);
edgeImg = skeleton;
edgeImg(poiCross) = 0;
[L, num] = bwlabel(edgeImg,8);

neighbors = [ 1,0 ; 0,1 ; -1,0 ; 0,-1 ];
[r,c] = find(poi);
labeledEdges = L;
pointsLabeled = zeros(m,n);

% Fill gaps between branch/end points and edges
for i=1:length(r),
    pointsLabeled(r(i),c(i)) = i;
    for k=1:4,
        gap = neighbors(k,:)+[r(i),c(i)];
        roi = [max(gap(1)-1,1), max(gap(2)-1,1), min(gap(1)+1,m), min(gap(2)+1,n)];
        edgeLabels = nonzeros(L(roi(1):roi(3),roi(2):roi(4))); % bottom pixel
        if ~isempty(edgeLabels), 
            labeledEdges(gap(1),gap(2)) = skeleton(gap(1),gap(2)) * edgeLabels(1); 
        end
    end
end

edges = cell(num,1);
points = cell(length(r),1);

% Retrieve connection between edges and branch points
for i=1:length(r),
    edgesOfPoi = nonzeros(labeledEdges(r(i)-1:r(i)+1,c(i)-1:c(i)+1));
    %disp([i,r(i),c(i),edgesOfPoi']);
    for k=1:length(edgesOfPoi),
       edges{edgesOfPoi(k)} = [edges{edgesOfPoi(k)},i]; 
    end
    points{i} = edgesOfPoi;
end

% Collapse point clusters to single points
CC = bwconncomp(poi,8);
numPixels = cellfun(@numel,CC.PixelIdxList);
ind = find(numPixels>1);
pixels = CC.PixelIdxList(ind);
deletedPoints = [];
for i=1:length(ind),
    pointCluster = pointsLabeled(pixels{i});
    edgeInd = cell2mat(points(pointCluster));
    for ii = 1:length(edgeInd),
        edges{edgeInd(ii)}(ismember(edges{edgeInd(ii)},pointCluster)) = pointCluster(1);
    end 
    deletedPoints = [deletedPoints,pointCluster(2:end)'];
end

for i=1:num,
    edges{i} = unique(edges{i});
end

adjacencyMatrix = zeros(length(points),length(points),3);
for e=1:length(edges),
    if (length(edges{e})==2),
        vesselRadius = mean(mean((labeledEdges==e).*distTrafo));
        adjacencyMatrix(edges{e}(1),edges{e}(2),:) = [e,nnz(labeledEdges==e),vesselRadius];
        adjacencyMatrix(edges{e}(2),edges{e}(1),:) = [e,nnz(labeledEdges==e),vesselRadius];
    end
end

nodes = [r,c];
nodes(deletedPoints,:) = [];
adjacencyMatrix(:,deletedPoints,:) = [];
adjacencyMatrix(deletedPoints,:,:) = [];

figure, imagesc(vesselImage), colormap(jet),
hold on, gplot(adjacencyMatrix(:,:,1),[nodes(:,2),nodes(:,1)],'-w'), daspect([1,1,1])
