function [ maculaPos, maculaRadius] = intensityAnalysis( image, vessels, estimatePos, ondiskDiameter, filename )

% Compute inpainting of image
inpaint = image;
inpaint(:,:,1) = removeVessels(image(:,:,1), vessels);
inpaint(:,:,2) = removeVessels(image(:,:,2), vessels);
inpaint(:,:,3) = removeVessels(image(:,:,3), vessels);
image = inpaint;

% Define ROI from last estimated Position and roiRadius
radius = round(ondiskDiameter);
roiRows = max(estimatePos(1)-radius,1) : min(estimatePos(1)+radius , size(image,1));
roiCols = max(estimatePos(2)-radius,1) : min(estimatePos(2)+radius , size(image,2)); 

% disp(filename);
% disp(['estimatePos ',num2str(estimatePos(1)),' , ',num2str(estimatePos(2))])
% disp(['ROIradius ',num2str(radius)])

nrOfEstimates = 14;
maculaPos = zeros(nrOfEstimates,2);
maculaRadius = zeros(nrOfEstimates,1);

m = size(image,1);
n = size(image,2);

%% Fast radial symmetry transform
        
    % Extract RGB channels and flip grayscale such that macula is bright
    red = image(roiRows,roiCols,1);
    red = max(red(:))-red;
    green = image(roiRows,roiCols,2);
    green = max(green(:))-green;
    blue = image(roiRows,roiCols,3);
    blue = max(blue(:))-blue;

    % Define radii that should be evaluated in FRST
    minMaxRadii = [round(ondiskDiameter/2 - 0.5*ondiskDiameter/2),round(ondiskDiameter/2 + 0.5*ondiskDiameter/2)];
    step = max(round((minMaxRadii(2)-minMaxRadii(1))/10),1);
    radii = minMaxRadii(1):step:minMaxRadii(2);

    % Compute center positions and optimal radius of bright regions via FRST for each RGB-channel
    [S_red,centers_red,bestIdx_red] = frst1(red, radii, 1);
    [S_green,centers_green,bestIdx_green] = frst1(green, radii, 1);        
    [S_blue,centers_blue,bestIdx_blue] = frst1(blue, radii, 1);

    % Collect and display information on best centers and radii for each color channel
    bestCirclesRGB = [centers_red(bestIdx_red,:),radii(bestIdx_red),max(max(S_red(:,:,bestIdx_red)));...
                      centers_green(bestIdx_green,:),radii(bestIdx_green),max(max(S_green(:,:,bestIdx_green)));...
                      centers_blue(bestIdx_blue,:),radii(bestIdx_blue),max(max(S_blue(:,:,bestIdx_blue)))];
%     disp(['red(x,y,r,max): ',num2str(bestCirclesRGB(1,:))])        
%     disp(['green(x,y,r,max): ',num2str(bestCirclesRGB(2,:))])      
%     disp(['blue(x,y,r,max): ',num2str(bestCirclesRGB(3,:))])  

    % Alternative 1: Compute weighted sum of best radius of each channel and sum up 
    % their corresponding radial symmetry constribution image to extract its maximum         
    maculaRadius(1) = sum(bestCirclesRGB(:,3).*bestCirclesRGB(:,4)) / sum(bestCirclesRGB(:,4));
    [~,nearestIdx] = min(abs(radii-maculaRadius(1)));
    S_rgb = S_red(:,:,nearestIdx) + S_green(:,:,nearestIdx) + S_blue(:,:,nearestIdx);
    [~,idx] = max(S_rgb(:));
    [maculaPos(1,1),maculaPos(1,2)] = ind2sub(size(S_rgb),idx);
    
    % Alternative 14: Mix of distance transform and fast radial symmetry transform
    S_rgb = scaledata(S_rgb,0,1);
    distMap = bwdist(vessels);
    distMap = scaledata(distMap(roiRows,roiCols),0,1);
    mix = S_rgb .* distMap;
    [~,idx] = max(mix(:));
    [maculaPos(14,1),maculaPos(14,2)] = ind2sub(size(mix),idx);

    % Alternative 2: Pick mean of maximum positions of each channel and 
    % compute mean of the rgb-radii for macula radius
    maculaPos(2,:) = round(mean(bestCirclesRGB(:,1:2),1));    
    maculaRadius(2) = mean(bestCirclesRGB(:,3));

    % Alternative 3: Compute weighted sum of all centers (for 
    % each radius and channel) and compute macula radius analog
    ind_red = sub2ind(size(S_red),centers_red(:,1),centers_red(:,2),(1:length(radii))');
    ind_green = sub2ind(size(S_green),centers_green(:,1),centers_green(:,2),(1:length(radii))');
    ind_blue = sub2ind(size(S_blue),centers_blue(:,1),centers_blue(:,2),(1:length(radii))');
    maculaPos(3,:) = sum(repmat(S_red(ind_red),1,2).*centers_red) + ...
                sum(repmat(S_green(ind_green),1,2).*centers_green) + ...
                sum(repmat(S_blue(ind_blue),1,2).*centers_blue);
    maculaPos(3,:) = round(maculaPos(3,:) / (sum(S_red(ind_red))+sum(S_green(ind_green))+sum(S_blue(ind_blue))));
    maculaRadius(3) = sum(S_red(ind_red).*radii') + sum(S_green(ind_green).*radii') + sum(S_blue(ind_blue).*radii');
    maculaRadius(3) = maculaRadius(3) / (sum(S_red(ind_red))+sum(S_green(ind_green))+sum(S_blue(ind_blue)));
    
    % Alternative 4: Compute mean of all centers (for each
    % radius and channel) and compute macula radius analog
    maculaPos(4,:) = round(mean([centers_red;centers_green;centers_blue],1));
    maculaRadius(4) = round(mean(radii));

    % Alternative 5: Compute median of best centers and radii
    maculaPos(5,:) = median(bestCirclesRGB(:,1:2),1);
    maculaRadius(5) = median(bestCirclesRGB(:,3));
    
    % Alternative 6: Compute median of all centers and radii
    maculaPos(6,:) = round(median([centers_red;centers_green;centers_blue],1));
    maculaRadius(6) = round(median(radii));

%% Template matching

    C = makecform('srgb2xyz');
    xyz = applycform(image,C);
    C = makecform('srgb2lab');
    lab = applycform(image,C);
    maculaTemplate = fspecial('gaussian',[m,n],ondiskDiameter/3);
    maculaTemplate = scaledata(maculaTemplate,0,1);
    ftemplate = fft2(maculaTemplate);
    
    % Alternative 7: Compute Maximum in Template Matching of Gaussian
    % Kernel in X-channel of XYZ-Colorspace Image
    img = double(xyz(:,:,1));
    img = scaledata(img,1,0);
    fimg = fft2(img);
    matching = fftshift(ifft2(ftemplate.*fimg));
    matching = matching(roiRows,roiCols);
    [~,ind] = max(matching(:));
    [maculaPos(7,1),maculaPos(7,2)] = ind2sub(size(matching),ind);
 
    % Alternative 8: Compute Maximum in Template Matching of Gaussian
    % Kernel in L-channel of LAB-Colorspace Image
    img = double(lab(:,:,1));
    img = scaledata(img,1,0);
    fimg = fft2(img);
    matching = fftshift(ifft2(ftemplate.*fimg));
    matching = matching(roiRows,roiCols);
    [~,ind] = max(matching(:));
    [maculaPos(8,1),maculaPos(8,2)] = ind2sub(size(matching),ind);

    % Alternative 9: Compute Maximum in Template Matching of Gaussian
    % Kernel in red-channel of RGB-Colorspace Image
    img = double(image(:,:,1));
    img = scaledata(img,1,0);
    fimg = fft2(img);
    matching = fftshift(ifft2(ftemplate.*fimg));
    matching = matching(roiRows,roiCols);
    [~,ind] = max(matching(:));
    [maculaPos(9,1),maculaPos(9,2)] = ind2sub(size(matching),ind);

%% Optical Density Measurement for Macular Pigments
        
    % Define mask for circle inside ROI
    [xx,yy] = ndgrid(roiRows,roiCols);
    circularMask = ((xx-estimatePos(1)).^2+(yy-estimatePos(2)).^2>(ondiskDiameter-2)^2) .* ...
                   ((xx-estimatePos(1)).^2+(yy-estimatePos(2)).^2<ondiskDiameter^2);
    % Compute normalization factor from reference pixels (pixels in 
    % 6 degree distance from the center of the selected macula region)
    referencePixelsBlue = double(image(roiRows,roiCols,3)).*circularMask;
    referencePixelsGreen = double(image(roiRows,roiCols,2)).*circularMask;
    Nfactor = nanmean(referencePixelsGreen(:))/nanmean(referencePixelsBlue(:));
    % Compute optical density
    green = double(image(roiRows,roiCols,2));
    blue = double(image(roiRows,roiCols,3));
    odmp = -log(Nfactor * blue./green);
    % Replace Inf values by noninfinity maximum of odmp
    odmpMax = max(odmp(~isinf(odmp)));
    odmp(isinf(odmp)) = odmpMax;
    % Normalize density values to fit range [0,1]
    odmp = scaledata(odmp,0,1);

    % Alternative 10: Fit polynomial curve to data and determine local
    % extrema while ignoring NaN values
    [x,y] = ndgrid(1:size(odmp,1),1:size(odmp,2));
    f = fit([x(~isnan(odmp)), y(~isnan(odmp))], odmp(~isnan(odmp)), 'poly22');
    c = coeffvalues(f);
    maculaPos(10,1) = round((2*c(3)*c(4)-c(5)*c(2))/(c(5)^2-4*c(4)*c(6)));
    maculaPos(10,2) = round((-2*c(6)*maculaPos(10,1)-c(3))/(c(5)));        
    
    % Alternative 11: Determine whether center contains high or low values
    % and compute centroid of center pixels
    [ro,co] = find(odmp.*(odmp>0.5));
    ro = ro-size(odmp,1)/2;
    co = co-size(odmp,2)/2;
    meanDistHighValues = mean(sqrt(ro.^2+co.^2));
    [ro,co] = find(odmp.*(odmp<=0.5));
    ro = ro-size(odmp,1)/2;
    co = co-size(odmp,2)/2;
    meanDistLowValues = mean(sqrt(ro.^2+co.^2));
    if (meanDistHighValues>meanDistLowValues), % center contains low values
        [ro,co,val] = find(odmp.*(odmp<=0.5));
        maculaPos(11,1) = round(sum(ro.*val)/sum(val));
        maculaPos(11,2) = round(sum(co.*val)/sum(val));
    else % center contains high values
        [ro,co,val] = find(odmp.*(odmp>0.5));
        maculaPos(11,1) = round(sum(ro.*val)/sum(val));
        maculaPos(11,2) = round(sum(co.*val)/sum(val)); 
    end
    
    % Alternative 12: Mean of alfternative 10 and 11
    maculaPos(12,:) = round((maculaPos(11,:)+maculaPos(10,:))/2);


%% Opening 

    % Alternative 13: Find maxima in all 3 channels of opened image and compare
    openedROI = imopen(image(roiRows,roiCols,:),strel('disk',7));   
    [r1,c1] = find(openedROI(:,:,1)==min(min(openedROI(:,:,1))));
    [r2,c2] = find(openedROI(:,:,2)==min(min(openedROI(:,:,2))));
    [r3,c3] = find(openedROI(:,:,3)==min(min(openedROI(:,:,3))));
    maculaPos(13,:) = round([mean([r1;r2;r3]),mean([c1;c2;c3])]);


%% Map macula Positions from ROI to original image
maculaPos = maculaPos + repmat([roiRows(1),roiCols(2)],nrOfEstimates,1);
end

