%% Parabola Function
function [X,Y] = parabolaPoints( a , b , xBounds , yBounds, step)

% Check for correct a
if a<0
   disp('Error: a has to be positive'); 
end


% Precompute trigonometric functions
si = sin(b);
co = cos(b);

xmin = xBounds(1);
xmax = xBounds(2);
ymin = yBounds(1);
ymax = yBounds(2);

xi=xmin:step:xmax;
yi=ymin:step:ymax;

X=[];
Y=[];

% Case-by-case analysis for absolute value function:

if (abs(si)<0.0001)
    
    X = [Y, (a*yi.^2)/co];    
    Y = [Y, yi];
    
    xInd = find(xi*co>=0);
    X = [X, xi(xInd), xi(xInd)];
    Y = [Y, sqrt((xi(xInd)*co)/a), -sqrt((xi(xInd)*co)/a)];
    
elseif (abs(co)<0.0001)
    
    X = [X, xi];
    Y = [Y, (a*xi.^2)/(-si)];
    
    yInd = find(-yi*si>=0);
    Y = [Y, yi(yInd), yi(yInd)];
    X = [X, sqrt((-yi(yInd)*si)/a), -sqrt((-yi(yInd)*si)/a)];

else

    % determine x from given y
    [Y1, X1] = solveQuadraticFunc(yi, a*si^2, 2*a*yi*si*co - co, a*yi.^2*co^2 + yi*si);
    X=[X,X1];
    Y=[Y,Y1];
    
    % determine y from given x
    [X1, Y1] = solveQuadraticFunc(xi, a*co^2, 2*a*xi*si*co + si, a*xi.^2*si^2 - xi*co);
    X=[X,X1];
    Y=[Y,Y1];
  
end

i = find((X<=xmax) .* (X>=xmin) .* (Y<=ymax) .* (Y>=ymin));
X = X(i);
Y = Y(i);

end

%% Helper Function
function [X,Y] = solveQuadraticFunc (x, c1,c2,c3)

    radicand = c2.^2 - 4*c1*c3;
    xInd = find(radicand>=0);
    
    y1 = (-c2(xInd)+sqrt(radicand(xInd)))/(2*c1);
    y2 = (-c2(xInd)-sqrt(radicand(xInd)))/(2*c1);
    
    X = [x(xInd),x(xInd)];    
    Y = [y1,y2]; 
end
